# -*- coding: utf-8 -*-
from gtts.tokenizer import pre_processors, Tokenizer, tokenizer_cases
from gtts.utils import _minimize, _len, _clean_tokens
from gtts.lang import tts_langs

from gtts_token import gtts_token
from six.moves import urllib
import urllib3
import requests
import logging

__all__ = ['gTTS', 'gTTSError']

# Logger
log = logging.getLogger(__name__)
log.addHandler(logging.NullHandler())


class Speed:
    """Read Speed

    The Google TTS Translate API supports two speeds:
        'slow' <= 0.3 < 'normal'
    """
    SLOW = 0.3
    NORMAL = 1


class gTTS:
    """gTTS -- Google Text-to-Speech.

    An interface to Google Translate's Text-to-Speech API.

    Args:
        text (string): The text to be read.
        lang (string, optional): The language (IETF language tag) to
            read the text in. Defaults to 'en'.
        slow (bool, optional): Reads text more slowly. Defaults to ``False``.
        lang_check (bool, optional): Strictly enforce an existing ``lang``,
            to catch a language error early. If set to ``True``,
            a ``ValueError`` is raised if ``lang`` doesn't exist.
            Default is ``True``.
        pre_processor_funcs (list): A list of zero or more functions that are
            called to transform (pre-process) text before tokenizing. Those
            functions must take a string and return a string. Defaults to::

                [
                    pre_processors.tone_marks,
                    pre_processors.end_of_line,
                    pre_processors.abbreviations,
                    pre_processors.word_sub
                ]

        tokenizer_func (callable): A function that takes in a string and
            returns a list of string (tokens). Defaults to::

                Tokenizer([
                    tokenizer_cases.tone_marks,
                    tokenizer_cases.period_comma,
                    tokenizer_cases.other_punctuation
                ]).run

    See Also:
        :doc:`Pre-processing and tokenizing <tokenizer>`

    Raises:
        AssertionError: When ``text`` is ``None`` or empty; when there's nothing
            left to speak after pre-precessing, tokenizing and cleaning.
        ValueError: When ``lang_check`` is ``True`` and ``lang`` is not supported.
        RuntimeError: When ``lang_check`` is ``True`` but there's an error loading
            the languages dictionnary.

    """

    GOOGLE_TTS_MAX_CHARS = 100  # Max characters the Google TTS API takes at a time
    GOOGLE_TTS_URL = "https://translate.google.com/translate_tts"
    GOOGLE_TTS_HEADERS = {
        "Referer": "http://translate.google.com/",
        "User-Agent":
            "Mozilla/5.0 (Windows NT 10.0; WOW64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/47.0.2526.106 Safari/537.36"
    }

    def __init__(
            self,
            text,
            lang='en',
            slow=False,
            lang_check=True,
            pre_processor_funcs=[
                pre_processors.tone_marks,
                pre_processors.end_of_line,
                pre_processors.abbreviations,
                pre_processors.word_sub
            ],
            tokenizer_func=Tokenizer([
                tokenizer_cases.tone_marks,
                tokenizer_cases.period_comma,
                tokenizer_cases.other_punctuation
            ]).run
    ):

        # Debug
        for k, v in locals().items():
            if k == 'self':
                continue
            log.debug("%s: %s", k, v)

        # Text
        assert text, 'No text to speak'
        self.text = text

        # Language
        if lang_check:
            try:
                langs = tts_langs()
                if lang.lower() not in langs:
                    raise ValueError("Language not supported: %s" % lang)
            except RuntimeError as e:
                log.debug(str(e), exc_info=True)
                log.warning(str(e))

        self.lang_check = lang_check
        self.lang = lang.lower()

        # Read speed
        if slow:
            self.speed = Speed.SLOW
        else:
            self.speed = Speed.NORMAL

        # Pre-processors and tokenizer
        self.pre_processor_funcs = pre_processor_funcs
        self.tokenizer_func = tokenizer_func

        # Google Translate token
        self.token = gtts_token.Token()

    def _tokenize(self, text):
        # Pre-clean
        text = text.strip()

        # Apply pre-processors
        for pp in self.pre_processor_funcs:
            log.debug("pre-processing: %s", pp)
            text = pp(text)

        if _len(text) <= self.GOOGLE_TTS_MAX_CHARS:
            return _clean_tokens([text])

        # Tokenize
        log.debug("tokenizing: %s", self.tokenizer_func)
        tokens = self.tokenizer_func(text)

        # Clean
        tokens = _clean_tokens(tokens)

        # Minimize
        min_tokens = []
        for t in tokens:
            min_tokens += _minimize(t, ' ', self.GOOGLE_TTS_MAX_CHARS)
        return min_tokens

    def write_to_fp(self, fp):
        """Do the TTS API request and write bytes to a file-like object.

        Args:
            fp (file object): Any file-like object to write the ``mp3`` to.

        Raises:
            :class:`gTTSError`: When there's an error with the API request.
            TypeError: When ``fp`` is a file-like object that takes bytes.

        """
        # When disabling ssl verify in requests (for proxies and firewalls),
        # urllib3 prints an insecure warning on stdout. We disable that.
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        text_parts = self._tokenize(self.text)
        log.debug("text_parts: %i", len(text_parts))
        assert text_parts, 'No text to send to TTS API'

        for idx, part in enumerate(text_parts):
            try:
                # Calculate token
                part_tk = self.token.calculate_token(part)
            except requests.exceptions.RequestException as e:  # pragma: no cover
                log.debug(str(e), exc_info=True)
                raise gTTSError(
                    "Connection error during token calculation: %s" %
                    str(e))

            payload = {'ie': 'UTF-8',
                       'q': part,
                       'tl': self.lang,
                       'ttsspeed': self.speed,
                       'total': len(text_parts),
                       'idx': idx,
                       'client': 'tw-ob',
                       'textlen': _len(part),
                       'tk': part_tk}

            log.debug("payload-%i: %s", idx, payload)

            try:
                # Request
                r = requests.get(self.GOOGLE_TTS_URL,
                                 params=payload,
                                 headers=self.GOOGLE_TTS_HEADERS,
                                 proxies=urllib.request.getproxies(),
                                 verify=False)

                log.debug("headers-%i: %s", idx, r.request.headers)
                log.debug("url-%i: %s", idx, r.request.url)
                log.debug("status-%i: %s", idx, r.status_code)

                r.raise_for_status()
            except requests.exceptions.HTTPError as e:
                # Request successful, bad response
                raise gTTSError(tts=self, response=r)
            except requests.exceptions.RequestException as e:  # pragma: no cover
                # Request failed
                raise gTTSError(str(e))

            try:
                # Write
                for chunk in r.iter_content(chunk_size=1024):
                    fp.write(chunk)
                log.debug("part-%i written to %s", idx, fp)
            except (AttributeError, TypeError) as e:
                raise TypeError(
                    "'fp' is not a file-like object or it does not take bytes: %s" %
                    str(e))

    def save(self, savefile):
        """Do the TTS API request and write result to file.

        Args:
            savefile (string): The path and file name to save the ``mp3`` to.

        Raises:
            :class:`gTTSError`: When there's an error with the API request.

        """
        with open(savefile, 'wb') as f:
            self.write_to_fp(f)
            log.debug("Saved to %s", savefile)


class gTTSError(Exception):
    """Exception that uses context to present a meaningful error message"""

    def __init__(self, msg=None, **kwargs):
        self.tts = kwargs.pop('tts', None)
        self.rsp = kwargs.pop('response', None)
        if msg:
            self.msg = msg
        elif self.tts is not None and self.rsp is not None:
            self.msg = self.infer_msg(self.tts, self.rsp)
        else:
            self.msg = None
        super(gTTSError, self).__init__(self.msg)

    def infer_msg(self, tts, rsp):
        """Attempt to guess what went wrong by using known
        information (e.g. http response) and observed behaviour

        """
        # rsp should be <requests.Response>
        # http://docs.python-requests.org/en/master/api/
        status = rsp.status_code
        reason = rsp.reason

        cause = "Unknown"
        if status == 403:
            cause = "Bad token or upstream API changes"
        elif status == 404 and not tts.lang_check:
            cause = "Unsupported language '%s'" % self.tts.lang
        elif status >= 500:
            cause = "Uptream API error. Try again later."

        return "%i (%s) from TTS API. Probable cause: %s" % (
            status, reason, cause)



# -*- coding: utf-8 -*-
from gtts import gTTS, gTTSError, __version__
from gtts.lang import tts_langs
import click
import locale
import logging
import logging.config

# Click settings
CONTEXT_SETTINGS = {
    'help_option_names': ['-h', '--help']
}

# Logger settings
LOGGER_SETTINGS = {
    'version': 1,
    'formatters': {
        'default': {
            'format': '%(name)s - %(levelname)s - %(message)s'
        }
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'default'
        }
    },
    'loggers': {
        'gtts': {
            'handlers': ['console'],
            'level': 'WARNING'
        }
    }
}

# Logger
logging.config.dictConfig(LOGGER_SETTINGS)
log = logging.getLogger('gtts')


def sys_encoding():
    """Charset to use for --file <path>|- (stdin)"""
    return 'utf8'


def validate_text(ctx, param, text):
    """Validation callback for the <text> argument.
    Ensures <text> (arg) and <file> (opt) are mutually exclusive
    """
    if not text and 'file' not in ctx.params:
        # No <text> and no <file>
        raise click.BadParameter(
            "<text> or -f/--file <file> required")
    if text and 'file' in ctx.params:
        # Both <text> and <file>
        raise click.BadParameter(
            "<text> and -f/--file <file> can't be used together")
    return text


def validate_lang(ctx, param, lang):
    """Validation callback for the <lang> option.
    Ensures <lang> is a supported language unless the <nocheck> flag is set
    """
    if ctx.params['nocheck']:
        return lang

    try:
        if lang not in tts_langs():
            raise click.UsageError(
                "'%s' not in list of supported languages.\n"
                "Use --all to list languages or "
                "add --nocheck to disable language check." % lang)
        else:
            # The language is valid.
            # No need to let gTTS re-validate.
            ctx.params['nocheck'] = True
    except RuntimeError as e:
        # Only case where the <nocheck> flag can be False
        # Non-fatal. gTTS will try to re-validate.
        log.debug(str(e), exc_info=True)

    return lang


def print_languages(ctx, param, value):
    """Callback for <all> flag.
    Prints formatted sorted list of supported languages and exits
    """
    if not value or ctx.resilient_parsing:
        return
    try:
        langs = tts_langs()
        langs_str_list = sorted("{}: {}".format(k, langs[k]) for k in langs)
        click.echo('  ' + '\n  '.join(langs_str_list))
    except RuntimeError as e:  # pragma: no cover
        log.debug(str(e), exc_info=True)
        raise click.ClickException("Couldn't fetch language list.")
    ctx.exit()


def set_debug(ctx, param, debug):
    """Callback for <debug> flag.
    Sets logger level to DEBUG
    """
    if debug:
        log.setLevel(logging.DEBUG)
    return


@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument('text', metavar='<text>', required=False, callback=validate_text)
@click.option(
    '-f',
    '--file',
    metavar='<file>',
    # For py2.7/unicode. If encoding not None Click uses io.open
    type=click.File(encoding=sys_encoding()),
    help="Read from <file> instead of <text>.")
@click.option(
    '-o',
    '--output',
    metavar='<file>',
    type=click.File(mode='wb'),
    help="Write to <file> instead of stdout.")
@click.option(
    '-s',
    '--slow',
    default=False,
    is_flag=True,
    help="Read more slowly.")
@click.option(
    '-l',
    '--lang',
    metavar='<lang>',
    default='en',
    show_default=True,
    callback=validate_lang,
    help="IETF language tag. Language to speak in. List documented tags with --all.")
@click.option(
    '--nocheck',
    default=False,
    is_flag=True,
    is_eager=True,  # Prioritize <nocheck> to ensure it gets set before <lang>
    help="Disable strict IETF language tag checking. Allow undocumented tags.")
@click.option(
    '--all',
    default=False,
    is_flag=True,
    is_eager=True,
    expose_value=False,
    callback=print_languages,
    help="Print all documented available IETF language tags and exit.")
@click.option(
    '--debug',
    default=False,
    is_flag=True,
    is_eager=True,  # Prioritize <debug> to see debug logs of callbacks
    expose_value=False,
    callback=set_debug,
    help="Show debug information.")
@click.version_option(version=__version__)
def tts_cli(text, file, output, slow, lang, nocheck):
    """ Read <text> to mp3 format using Google Translate's Text-to-Speech API 
    (set <text> or --file <file> to - for standard input)
    """

    # stdin for <text>
    if text == '-':
        text = click.get_text_stream('stdin').read()

    # stdout (when no <output>)
    if not output:
        output = click.get_binary_stream('stdout')

    # <file> input (stdin on '-' is handled by click.File)
    if file:
        try:
            text = file.read()
        except UnicodeDecodeError as e:  # pragma: no cover
            log.debug(str(e), exc_info=True)
            raise click.FileError(
                file.name,
                "<file> must be encoded using '%s'." %
                sys_encoding())

    # TTS
    try:
        tts = gTTS(
            text=text,
            lang=lang,
            slow=slow,
            lang_check=not nocheck)
        tts.write_to_fp(output)
    except (ValueError, AssertionError) as e:
        raise click.UsageError(str(e))
    except gTTSError as e:
        raise click.ClickException(str(e))



# -*- coding: utf-8 -*-
import unittest
from gtts.utils import _minimize, _len, _clean_tokens


class TestMinimize(unittest.TestCase):
    def setUp(self):
        self.delim = ' '
        self.max = 10

    def test_ascii(self):
        _in = "Bacon ipsum dolor sit amet"
        _out = ["Bacon", "ipsum", "dolor sit", "amet"]
        self.assertEqual(_minimize(_in, self.delim, self.max), _out)

    def test_ascii_no_delim(self):
        _in = "Baconipsumdolorsitametflankcornedbee"
        _out = ["Baconipsum", "dolorsitam", "etflankcor", "nedbee"]
        self.assertEqual(_minimize(_in, self.delim, self.max), _out)

    def test_unicode(self):
        _in = u"这是一个三岁的小孩在讲述他从一系列照片里看到的东西。"
        _out = [u"这是一个三岁的小孩在", u"讲述他从一系列照片里", u"看到的东西。"]
        self.assertEqual(_minimize(_in, self.delim, self.max), _out)

    def test_startwith_delim(self):
        _in = self.delim + "test"
        _out = ["test"]
        self.assertEqual(_minimize(_in, self.delim, self.max), _out)


class TestLen(unittest.TestCase):
    def test_ascii(self):
        text = "Bacon ipsum dolor sit amet flank corned beef."
        self.assertEqual(_len(text), 45)

    def test_unicode(self):
        text = u"但在一个重要的任务上"
        self.assertEqual(_len(text), 10)


class TestCleanToken(unittest.TestCase):
    def test_only_space_and_punc(self):
        _in = [",(:)?", "\t    ", "\n"]
        _out = []
        self.assertEqual(_clean_tokens(_in), _out)

    def test_strip(self):
        _in = [" Bacon  ", "& ", "ipsum\r", "."]
        _out = ["Bacon", "&", "ipsum"]
        self.assertEqual(_clean_tokens(_in), _out)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import tempfile
import unittest
import click
import six
import sys
import os
from click.testing import CliRunner
from gtts.cli import tts_cli

# Need to look into gTTS' log output to test proper instanciation
# - Use testfixtures.LogCapture() b/c TestCase.assertLogs() needs py3.4+
# - Clear 'gtts' logger handlers (set in gtts.cli) to reduce test noise
import logging
from testfixtures import LogCapture
logger = logging.getLogger('gtts')
logger.handlers = []


class TestParams(unittest.TestCase):
    """Test options and arguments"""

    @classmethod
    def setUpClass(self):
        self.runner = CliRunner()
        (_, self.empty_file_path) = tempfile.mkstemp(suffix='.txt')

    def invoke(self, args, input=None):
        return self.runner.invoke(tts_cli, args, input)

    def invoke_debug(self, args, input=None):
        all_args = args + ['--debug']
        return self.invoke(all_args, input)

    # <text> tests
    def test_text_no_text_or_file(self):
        """One of <test> (arg) and <file> <opt> should be set"""
        result = self.invoke_debug([])

        self.assertIn("<file> required", result.output)
        self.assertNotEqual(result.exit_code, 0)

    def test_text_text_and_file(self):
        """<test> (arg) and <file> <opt> should not be set together"""
        result = self.invoke_debug(['--file', self.empty_file_path, 'test'])

        self.assertIn("<file> can't be used together", result.output)
        self.assertNotEqual(result.exit_code, 0)

    def test_text_empty(self):
        """Exit on no text to speak (via <file>)"""
        result = self.invoke_debug(['--file', self.empty_file_path])

        self.assertIn("No text to speak", result.output)
        self.assertNotEqual(result.exit_code, 0)

    # <file> tests
    def test_file_not_exists(self):
        """<file> should exist"""
        result = self.invoke_debug(['--file', 'notexist.txt', 'test'])

        self.assertIn("No such file or directory", result.output)
        self.assertNotEqual(result.exit_code, 0)

    # <all> tests
    def test_all(self):
        """Option <all> should return a list of languages"""
        result = self.invoke(['--all'])

        # One or more of "  xy: name" (\n optional to match the last)
        # Ex. "<start>  xx: xxxxx\n  xx-yy: xxxxx\n  xx: xxxxx<end>"
        # NB: assertRegex needs Py3.1+, use six
        six.assertRegex(
            self,
            result.output,
            "^(?:\s{2}(\w{2}|\w{2}-\w{2}): .+\n?)+$")
        self.assertEqual(result.exit_code, 0)

    # <lang> tests
    def test_lang_not_valid(self):
        """Invalid <lang> should display an error"""
        result = self.invoke(['--lang', 'xx', 'test'])

        self.assertIn("xx' not in list of supported languages", result.output)
        self.assertNotEqual(result.exit_code, 0)

    def test_lang_nocheck(self):
        """Invalid <lang> (with <nocheck>) should display an error message from gtts"""
        with LogCapture() as lc:
            result = self.invoke_debug(
                ['--lang', 'xx', '--nocheck', 'test'])

            log = str(lc)

        self.assertIn('lang: xx', log)
        self.assertIn('lang_check: False', log)
        self.assertIn(
            "Probable cause: Unsupported language 'xx'",
            result.output)
        self.assertNotEqual(result.exit_code, 0)

    # Param set tests
    def test_params_set(self):
        """Options should set gTTS instance arguments (read from debug log)"""
        with LogCapture() as lc:
            result = self.invoke_debug(
                ['--lang', 'fr', '--slow', '--nocheck', 'test'])

            log = str(lc)

        self.assertIn('lang: fr', log)
        self.assertIn('lang_check: False', log)
        self.assertIn('slow: True', log)
        self.assertIn('text: test', log)
        self.assertEqual(result.exit_code, 0)


class TestInputs(unittest.TestCase):
    """Test all input methods"""

    @classmethod
    def setUpClass(self):
        self.runner = CliRunner()

    def setUp(self):
        pwd = os.path.dirname(__file__)

        # Text for stdin ('-' for <text> or <file>)
        self.textstdin = """stdin
test
123"""

        # Text for stdin ('-' for <text> or <file>) (Unicode)
        self.textstdin_unicode = u"""你吃饭了吗？
你最喜欢哪部电影？
我饿了，我要去做饭了。"""

        # Text for <text> and <file>
        self.text = """Can you make pink a little more pinkish can you make pink a little more pinkish, nor can you make the font bigger?
How much will it cost the website doesn't have the theme i was going for."""
        self.textfile_ascii = os.path.join(
            pwd, 'input_files', 'test_cli_test_ascii.txt')

        # Text for <text> and <file> (Unicode)
        self.text_unicode = u"""这是一个三岁的小孩
在讲述她从一系列照片里看到的东西。
对这个世界， 她也许还有很多要学的东西，
但在一个重要的任务上， 她已经是专家了：
去理解她所看到的东西。"""
        self.textfile_utf8 = os.path.join(
            pwd, 'input_files', 'test_cli_test_utf8.txt')

    def invoke(self, args, input=None):
        return self.runner.invoke(tts_cli, args, input)

    def invoke_debug(self, args, input=None):
        all_args = args + ['--debug']
        return self.invoke(all_args, input)

    # Method that mimics's LogCapture's __str__ method to make
    # the string in the comprehension a unicode literal for P2.7
    # https://github.com/Simplistix/testfixtures/blob/32c87902cb111b7ede5a6abca9b597db551c88ef/testfixtures/logcapture.py#L149
    def logcapture_str(self, lc):
        if not lc.records:
            return 'No logging captured'
        return '\n'.join([u"%s %s\n  %s" % r for r in lc.actual()])

    def test_stdin_text(self):
        with LogCapture() as lc:
            result = self.invoke_debug(['-'], self.textstdin)
            log = self.logcapture_str(lc)

        self.assertIn('text: %s' % self.textstdin, log)
        self.assertEqual(result.exit_code, 0)

    def test_stdin_text_unicode(self):
        with LogCapture() as lc:
            result = self.invoke_debug(['-'], self.textstdin_unicode)
            log = self.logcapture_str(lc)

        self.assertIn(u'text: %s' % self.textstdin_unicode, log)
        self.assertEqual(result.exit_code, 0)

    def test_stdin_file(self):
        with LogCapture() as lc:
            result = self.invoke_debug(['--file', '-'], self.textstdin)
            log = self.logcapture_str(lc)

        self.assertIn('text: %s' % self.textstdin, log)
        self.assertEqual(result.exit_code, 0)

    def test_stdin_file_unicode(self):
        with LogCapture() as lc:
            result = self.invoke_debug(['--file', '-'], self.textstdin_unicode)
            log = self.logcapture_str(lc)

        self.assertIn('text: %s' % self.textstdin_unicode, log)
        self.assertEqual(result.exit_code, 0)

    def test_text(self):
        with LogCapture() as lc:
            result = self.invoke_debug([self.text])
            log = self.logcapture_str(lc)

        self.assertIn("text: %s" % self.text, log)
        self.assertEqual(result.exit_code, 0)

    def test_text_unicode(self):
        with LogCapture() as lc:
            result = self.invoke_debug([self.text_unicode])
            log = self.logcapture_str(lc)

        self.assertIn("text: %s" % self.text_unicode, log)
        self.assertEqual(result.exit_code, 0)

    def test_file_ascii(self):
        with LogCapture() as lc:
            result = self.invoke_debug(['--file', self.textfile_ascii])
            log = self.logcapture_str(lc)

        self.assertIn("text: %s" % self.text, log)
        self.assertEqual(result.exit_code, 0)

    def test_file_utf8(self):
        with LogCapture() as lc:
            result = self.invoke_debug(['--file', self.textfile_utf8])
            log = self.logcapture_str(lc)

        self.assertIn("text: %s" % self.text_unicode, log)
        self.assertEqual(result.exit_code, 0)


class TestOutputs(unittest.TestCase):
    """Test all ouput methods"""

    @classmethod
    def setUp(self):
        self.runner = CliRunner()
        (_, self.save_file_path) = tempfile.mkstemp(suffix='.mp3')

    def invoke(self, args, input=None):
        return self.runner.invoke(tts_cli, args, input)

    def test_stdout(self):
        result = self.invoke(['test'])

        # The MP3 encoding (LAME 3.99.5) leaves a signature in the raw output
        self.assertIn('LAME3.99.5', result.output)
        self.assertEqual(result.exit_code, 0)

    def test_file(self):
        result = self.invoke(['test', '--output', self.save_file_path])

        # Check if files created is > 2k
        self.assertTrue(os.path.getsize(self.save_file_path) > 2000)
        self.assertEqual(result.exit_code, 0)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import os
import io
import tempfile
import unittest
from mock import Mock

from gtts.tts import gTTS, gTTSError
from gtts.lang import tts_langs, _fetch_langs, _extra_langs

# Testing all languages takes some time.
# Set TEST_LANGS envvar to choose languages to test.
#  * 'fetch': Languages fetched from the Web
#  * 'extra': Languagee set in Languages.EXTRA_LANGS
#  * 'all': All of the above
#  * <csv>: Languages tags list to test
# Unset TEST_LANGS to test everything ('all')
# See: test_langs_dict()


class TestTTS(unittest.TestCase):
    """Test all supported languages and file save"""

    def setUp(self):
        self.text = "This is a test"

    def check_tts(self, lang):
        """Create output .mp3 file successfully"""
        for slow in (False, True):
            with tempfile.SpooledTemporaryFile(suffix='.mp3', prefix='test_{}_'.format(lang)) as f:
                # Create gTTS and save
                tts = gTTS(self.text, lang, slow=slow)
                tts.write_to_fp(f)

                # Check if files created is > 2k
                self.assertTrue(os.fstat(f.fileno()).st_size > 2000)


def test_langs_dict():
    """Construct a dict of suites of languages to test.
    { '<suite name>' : <list or dict of language tags> }

    ex.: { 'fetch' : {'en': 'English', 'fr': 'French'},
           'extra' : {'en': 'English', 'fr': 'French'} }
    ex.: { 'environ' : ['en', 'fr'] }
    """
    langs = dict()
    env = os.environ.get('TEST_LANGS', '')
    if env == '' or env == 'all':
        langs['fetch'] = _fetch_langs()
        langs['extra'] = _extra_langs()
    elif env == 'fetch':
        langs['fetch'] = _fetch_langs()
    elif env == 'extra':
        langs['extra'] = _extra_langs()
    else:
        env_langs = env.split(',')
        env_langs = [l for l in env_langs if l]
        langs['environ'] = env_langs
    return langs


# Generate TestTTS.check_tts tests for each language
# Based on: http://stackoverflow.com/a/1194012
for suite, langs in test_langs_dict().items():
    for l in langs:
        def ch(l):
            return lambda self: self.check_tts(l)
        setattr(TestTTS, "test_tts_%s_%s" % (suite, l), ch(l))


class TestInit(unittest.TestCase):
    """Test gTTS init"""

    def test_unsupported_language_check(self):
        """Raise ValueError on unsupported language (with language check)"""
        lang = 'xx'
        text = "Lorem ipsum"
        check = True
        with self.assertRaises(ValueError):
            tts = gTTS(text=text, lang=lang, lang_check=check)

    def test_empty_string(self):
        """Raise AssertionError on empty string"""
        text = ""
        with self.assertRaises(AssertionError):
            tts = gTTS(text=text)

    def test_no_text_parts(self):
        """Raises AssertionError on no content to send to API (no text_parts)"""
        text = "                                                                                                          ..,\n"
        with self.assertRaises(AssertionError):
            with tempfile.SpooledTemporaryFile() as f:
                tts = gTTS(text=text)
                tts.write_to_fp(f)


class TestWrite(unittest.TestCase):
    """Test write_to_fp()/save() cases not covered elsewhere in this file"""

    def test_bad_fp_type(self):
        """Raise TypeError if fp is not a file-like object (no .write())"""
        # Create gTTS and save
        tts = gTTS(text='test')
        with self.assertRaises(TypeError):
            tts.write_to_fp(5)

    def test_save(self):
        """Save .mp3 file successfully"""
        (_, save_file_path) = tempfile.mkstemp(suffix='.mp3')

        # Create gTTS and save
        tts = gTTS(text='test')
        tts.save(save_file_path)

        # Check if file created is > 2k
        self.assertTrue(os.stat(save_file_path).st_size > 2000)


class TestgTTSError(unittest.TestCase):
    """Test gTTsError internal exception handling"""

    def test_msg(self):
        """Set exception message successfully"""
        error1 = gTTSError('test')
        self.assertEqual('test', error1.msg)

        error2 = gTTSError()
        self.assertIsNone(error2.msg)

    def test_infer_msg(self):
        """Infer message sucessfully based on context"""

        # 403
        tts403 = Mock()
        response403 = Mock(status_code=403, reason='aaa')
        error403 = gTTSError(tts=tts403, response=response403)
        self.assertEqual(
            error403.msg,
            "403 (aaa) from TTS API. Probable cause: Bad token or upstream API changes")

        # 404 (and not lang_check)
        tts404 = Mock(lang='xx', lang_check=False)
        response404 = Mock(status_code=404, reason='bbb')
        error404 = gTTSError(tts=tts404, response=response404)
        self.assertEqual(
            error404.msg,
            "404 (bbb) from TTS API. Probable cause: Unsupported language 'xx'")

        # >= 500
        tts500 = Mock()
        response500 = Mock(status_code=500, reason='ccc')
        error500 = gTTSError(tts=tts500, response=response500)
        self.assertEqual(
            error500.msg,
            "500 (ccc) from TTS API. Probable cause: Uptream API error. Try again later.")

        # Unknown (ex. 100)
        tts100 = Mock()
        response100 = Mock(status_code=100, reason='ddd')
        error100 = gTTSError(tts=tts100, response=response100)
        self.assertEqual(
            error100.msg,
            "100 (ddd) from TTS API. Probable cause: Unknown")


class TestWebRequest(unittest.TestCase):
    """Test Web Requests"""

    def setUp(self):
        self.text = "Lorem ipsum"

    def test_unsupported_language_no_check(self):
        """Raise gTTSError on unsupported language (without language check)"""
        lang = 'xx'
        check = False
        with self.assertRaises(gTTSError):
            with tempfile.SpooledTemporaryFile() as f:
                 # Create gTTS
                tts = gTTS(text=self.text, lang=lang, lang_check=check)
                tts.write_to_fp(f)


if __name__ == '__main__':
    unittest.main()
